// ________________________________________________________________________________________________________
// Copyright (c) 2020 GebraBit Inc. All rights reserved.
//
// This software, related documentation and any modifications thereto (collectively �Software�) is subject
// to GebraBit and its licensors' intellectual property rights under U.S. and international copyright
// and other intellectual property rights laws. 
//
// GebraBit and its licensors retain all intellectual property and proprietary rights in and to the Software
// and any use, reproduction, disclosure or distribution of the Software without an express license agreement
// from GebraBit is strictly prohibited.
 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT IN  
// NO EVENT SHALL GebraBit BE LIABLE FOR ANY DIRECT, SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, 
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
// NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
// OF THE SOFTWARE.
// @Author       	: Sepehr Azimi
// inspired by ADS1292R ProtoCentral source code
// ________________________________________________________________________________________________________

#include "Arduino.h"
#include "GebraBit_ADS1194.h"
#include <SPI.h>

int j,i;

volatile byte SPI_RX_Buff[15] ;
volatile static int SPI_RX_Buff_Count = 0;
volatile char *SPI_RX_Buff_Ptr;
volatile bool ads1194dataReceived = false;

uint16_t uecgtemp = 0;
uint16_t resultTemp = 0;

int16_t secgtemp = 0;

int16_t statusByte=0;

uint8_t LeadStatus=0;

boolean ads1194::getAds1194EcgSamples(const int dataReady,const int chipSelect,ads1194OutputValues *ecgRespirationValues)
{

  if ((digitalRead(dataReady)) == LOW)      // Sampling rate is set to 125SPS ,DRDY ticks for every 8ms
  {
    SPI_RX_Buff_Ptr = ads1194ReadData(chipSelect); // Read the data,point the data to a pointer

    for (int i = 0; i < 11; i++)
    {
      SPI_RX_Buff[SPI_RX_Buff_Count++] = *(SPI_RX_Buff_Ptr + i);  // store the result data in array
    }

    ads1194dataReceived = true;
    j = 0;

    for (i = 3; i < 11; i += 2)        
    {
      uecgtemp = (uint16_t) (  ((uint16_t)SPI_RX_Buff[i + 0] << 8) | ( (uint16_t) SPI_RX_Buff[i + 1]));
      secgtemp = (int16_t) (uecgtemp);

      (ecgRespirationValues->sDaqVals)[j++] = secgtemp;  
    }

    ecgRespirationValues->sresultTempResp = (int16_t)(resultTemp);
    ecgRespirationValues->leadoffDetected  = false;
	ads1194dataReceived = false;
    SPI_RX_Buff_Count = 0;
    return true;
  }

  else
    return false;
}

char* ads1194::ads1194ReadData(const int chipSelect)
{
  static char SPI_Dummy_Buff[12];
  digitalWrite(chipSelect, LOW);

  for (int i = 0; i < 11; ++i)
  {
    SPI_Dummy_Buff[i] = SPI.transfer(CONFIG_SPI_MASTER_DUMMY);
  }

  digitalWrite(chipSelect, HIGH);
  return SPI_Dummy_Buff;
}

void ads1194::ads1194Init(const int chipSelect,const int pwdnPin,const int startPin)
{
  ads1194Reset(pwdnPin);
  delay(100);
  ads1194DisableStart(startPin);
  ads1194EnableStart(startPin);
  ads1194HardStop(startPin);
  ads1194StartDataConvCommand(chipSelect);
  ads1194SoftStop(chipSelect);
  delay(50);
  ads1194StopReadDataContinuous(chipSelect);					// SDATAC command
  delay(300);
  ads1194RegWrite(ADS1194_REG_CONFIG1,0b00000110,chipSelect); 		//Set sampling rate to 125 SPS
  delay(10);
  ads1194RegWrite(ADS1194_REG_CONFIG2, 0b00100000,chipSelect);	//Lead-off comp off, test signal disabled
  delay(10);
  ads1194RegWrite(ADS1194_REG_CONFIG3, 0b11001100,chipSelect);		
  delay(10);
  ads1194RegWrite(ADS1194_REG_LOFF, 0b00000000,chipSelect);	
  delay(10);
  ads1194RegWrite(ADS1194_REG_CH1SET, 0b01100000,chipSelect);	
  delay(10);
  ads1194RegWrite(ADS1194_REG_CH2SET, 0b01100000,chipSelect);	
  delay(10);
  ads1194RegWrite(ADS1194_REG_CH3SET, 0b01100000,chipSelect);	
  delay(10);
  ads1194RegWrite(ADS1194_REG_CH4SET, 0b01100000,chipSelect);	
  delay(10);
  ads1194RegWrite(ADS1194_REG_RLDSENSP, 0b00000111,chipSelect);	
  delay(10);
  ads1194RegWrite(ADS1194_REG_RLDSENSN, 0b00000111,chipSelect);	
  delay(10);
  ads1194RegWrite(ADS1194_REG_LOFFSENSP, 0b00000000,chipSelect);	
  delay(10);
  ads1194RegWrite(ADS1194_REG_LOFFSENSN, 0b00000000,chipSelect);	
  delay(10);
  ads1194RegWrite(ADS1194_REG_LOFFFLIP, 0b00000000,chipSelect);	
  delay(10);
  ads1194RegWrite(ADS1194_REG_PACE, 0b00000000,chipSelect);		
  delay(10);
  ads1194RegWrite(0x16, 0b00000000,chipSelect);		
  delay(10);
  ads1194RegWrite(ADS1194_REG_CONFIG4, 0b00000000,chipSelect);		
  delay(10);
  ads1194RegWrite(ADS1194_REG_WCT1, 0b00001001,chipSelect);		
  delay(10);
  ads1194RegWrite(ADS1194_REG_WCT2, 0b11101100,chipSelect);		
  delay(10);

  ads1194StartReadDataContinuous(chipSelect);
  delay(10);
  ads1194EnableStart(startPin);
  //return readdata;
}

void ads1194::ads1194Reset(const int pwdnPin)
{
  digitalWrite(pwdnPin, HIGH);
  delay(100);					// Wait 100 mSec
  digitalWrite(pwdnPin, LOW);
  delay(100);
  digitalWrite(pwdnPin, HIGH);
  delay(100);
}

void ads1194::ads1194DisableStart(const int startPin)
{
  digitalWrite(startPin, LOW);
  delay(20);
}

void ads1194::ads1194EnableStart(const int startPin)
{
  digitalWrite(startPin, HIGH);
  delay(20);
}

void ads1194::ads1194HardStop (const int startPin)
{
  digitalWrite(startPin, LOW);
  delay(100);
}

void ads1194::ads1194StartDataConvCommand (const int chipSelect)
{
  ads1194SPICommandData(START,chipSelect);					// Send 0x08 to the ADS1x9x
}

void ads1194::ads1194SoftStop (const int chipSelect)
{
  ads1194SPICommandData(STOP,chipSelect);           // Send 0x0A to the ADS1x9x
}

void ads1194::ads1194StartReadDataContinuous (const int chipSelect)
{
  ads1194SPICommandData(RDATAC,chipSelect);					// Send 0x10 to the ADS1x9x
}

void ads1194::ads1194StopReadDataContinuous (const int chipSelect)
{
  ads1194SPICommandData(SDATAC,chipSelect);					// Send 0x11 to the ADS1x9x
}

void ads1194::ads1194SPICommandData(unsigned char dataIn,const int chipSelect)
{
  byte data[1];
  //data[0] = dataIn;
  digitalWrite(chipSelect, LOW);
  delay(2);
  digitalWrite(chipSelect, HIGH);
  delay(2);
  digitalWrite(chipSelect, LOW);
  delay(2);
  SPI.transfer(dataIn);
  delay(2);
  digitalWrite(chipSelect, HIGH);
}

//Sends a write command to SCP1000
void ads1194::ads1194RegWrite (unsigned char READ_WRITE_ADDRESS, unsigned char DATA,const int chipSelect)
{
  byte dataToSend = READ_WRITE_ADDRESS | WREG;
  digitalWrite(chipSelect, LOW);
  delay(2);
  digitalWrite(chipSelect, HIGH);
  delay(2);
  // take the chip select low to select the device:
  digitalWrite(chipSelect, LOW);
  delay(2);
  SPI.transfer(dataToSend); //Send register location
  SPI.transfer(0x00);		//number of register to wr
  SPI.transfer(DATA);		//Send value to record into register
  delay(2);
  digitalWrite(chipSelect, HIGH);
}

unsigned char ads1194::ads1194RegRead (unsigned char READ_WRITE_ADDRESS,const int chipSelect)
{

  unsigned char data;
  // now combine the register address and the command into one byte:
  byte dataToSend = READ_WRITE_ADDRESS | RREG;
  digitalWrite(chipSelect, LOW);
  delay(2);
  digitalWrite(chipSelect, HIGH);
  delay(2);
  // take the chip select low to select the device:
  digitalWrite(chipSelect, LOW);
  delay(2);
  SPI.transfer(dataToSend); //Send register location
  SPI.transfer(0x00);		//number of register to wr
  //data= SPI.transfer(0x00);		//number of register to wr
  data=SPI.transfer(CONFIG_SPI_MASTER_DUMMY);		//Send value to record into register
  delay(2);
  // take the chip select high to de-select:
  digitalWrite(chipSelect, HIGH);
  return data;
}

