// ________________________________________________________________________________________________________
// Copyright (c) 2020 GebraBit Inc. All rights reserved.
//
// This software, related documentation and any modifications thereto (collectively �Software�) is subject
// to GebraBit and its licensors' intellectual property rights under U.S. and international copyright
// and other intellectual property rights laws. 
//
// GebraBit and its licensors retain all intellectual property and proprietary rights in and to the Software
// and any use, reproduction, disclosure or distribution of the Software without an express license agreement
// from GebraBit is strictly prohibited.
 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT IN  
// NO EVENT SHALL GebraBit BE LIABLE FOR ANY DIRECT, SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, 
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
// NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
// OF THE SOFTWARE.
// @Author       	: Sepehr Azimi
// inspired by ADS1292R ProtoCentral source code
// ________________________________________________________________________________________________________

#ifndef ads1194_h
#define ads1194_h

#include "Arduino.h"

#define CONFIG_SPI_MASTER_DUMMY   0xFF

// Register Read Commands
#define RREG                      0x20;		//Read n nnnn registers starting at address r rrrr
                                            //first byte 001r rrrr (2xh)(2) - second byte 000n nnnn(2)
#define WREG                      0x40;	    //Write n nnnn registers starting at address r rrrr
                                            //first byte 010r rrrr (2xh)(2) - second byte 000n nnnn(2)
#define WAKEUP	                  0x02
#define STANDBY                   0x04
#define RESET	                    0x06
#define START   	                0x08
#define STOP     	                0x0A
#define OFFSETCAL	                0x08					  
#define RDATAC                    0x10		//Enable Read Data Continuous mode.

//This mode is the default mode at power-up.
#define SDATAC                    0x11		//Stop Read Data Continuously mode
#define RDATA	                    0x12		//Read data by command; supports multiple read back.

//register address
#define ADS1194_REG_ID			      0x00
#define ADS1194_REG_CONFIG1		    0x01
#define ADS1194_REG_CONFIG2		    0x02
#define ADS1194_REG_CONFIG3		    0x03
#define ADS1194_REG_LOFF		      0x04
#define ADS1194_REG_CH1SET	 	    0x05
#define ADS1194_REG_CH2SET		    0x06
#define ADS1194_REG_CH3SET	 	    0x07
#define ADS1194_REG_CH4SET		    0x08
#define ADS1194_REG_RLDSENSP	    0x0D
#define ADS1194_REG_RLDSENSN	    0x0E
#define ADS1194_REG_LOFFSENSP     0x0F
#define ADS1194_REG_LOFFSENSN     0x10
#define ADS1194_REG_LOFFFLIP      0x11
#define ADS1194_REG_LOFFSTATP     0x12
#define ADS1194_REG_LOFFSTATN     0x13
#define ADS1194_REG_GPIO	        0x14
#define ADS1194_REG_PACE	        0x15
#define ADS1194_REG_CONFIG4		    0x17
#define ADS1194_REG_WCT1		      0x18
#define ADS1194_REG_WCT2		      0x19
//Packet format
#define  CES_CMDIF_PKT_START_1    0x0A
#define  CES_CMDIF_PKT_START_2    0xFA
#define  CES_CMDIF_TYPE_DATA      0x02
#define  CES_CMDIF_PKT_STOP_1     0x00
#define  CES_CMDIF_PKT_STOP_2     0x0B

typedef struct Record{
  volatile int16_t sDaqVals[10];
  boolean leadoffDetected = true;
  int16_t sresultTempResp;
}ads1194OutputValues;

class ads1194
{
  public:
    boolean getAds1194EcgSamples(const int dataReady,const int chipSelect,ads1194OutputValues *ecgRespirationValues);
    static void ads1194Init(const int chipSelect,const int pwdnPin,const int startPin);
    static void ads1194Reset(const int pwdnPin);
	

  private:
	  static void ads1194RegWrite (unsigned char READ_WRITE_ADDRESS, unsigned char DATA,const int chipSelect);
	  static unsigned char ads1194RegRead  (unsigned char READ_WRITE_ADDRESS,const int chipSelect);
    static void ads1194SPICommandData(unsigned char dataIn,const int chipSelect);
    static void ads1194DisableStart(const int startPin);
    static void ads1194EnableStart(const int startPin);
    static void ads1194HardStop (const int startPin);
    static void ads1194StartDataConvCommand (const int chipSelect);
    static void ads1194SoftStop (const int chipSelect);
    static void ads1194StartReadDataContinuous (const int chipSelect);
    static void ads1194StopReadDataContinuous (const int chipSelect);
    static char* ads1194ReadData(const int chipSelect);
};

#endif
