// ________________________________________________________________________________________________________
// Copyright (c) 2020 GebraBit Inc. All rights reserved.
//
// This software, related documentation and any modifications thereto (collectively �Software�) is subject
// to GebraBit and its licensors' intellectual property rights under U.S. and international copyright
// and other intellectual property rights laws. 
//
// GebraBit and its licensors retain all intellectual property and proprietary rights in and to the Software
// and any use, reproduction, disclosure or distribution of the Software without an express license agreement
// from GebraBit is strictly prohibited.
 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
// NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT IN  
// NO EVENT SHALL GebraBit BE LIABLE FOR ANY DIRECT, SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, 
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
// NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
// OF THE SOFTWARE.
// @Author       	: Sepehr Azimi
// inspired by ADS1292R ProtoCentral source code
// ________________________________________________________________________________________________________
//
//  |ads1194 pin label | Arduino Connection   |Pin Function      |
//  |----------------- |:--------------------:|-----------------:|
//  | VDD              | +3V3                 |  Supply voltage  |
//  | RST              | D7                   |  Reset           |
//  | STA              | A5                   |  Start Input     |
//  | RDY              | D2                   |  Data Ready Outpt|
//  | CS               | D10                  |  Chip Select     |
//  | SDI              | D11                  |  Slave In        |
//  | SDO              | D12                  |  Slave Out       |
//  | SCK              | D13                  |  Serial Clock    |
//  | GND              | Gnd                  |  Gnd             |
//
// ________________________________________________________________________________________________________

#include "GebraBit_ADS1194.h"
#include "ecgAlgo1194.h"
#include <SPI.h>

volatile uint8_t globalHeartRate = 0;

const int ADS1194_DRDY_PIN = 2;
const int ADS1194_CS_PIN = 10;
const int ADS1194_START_PIN = A5;
const int ADS1194_PWDN_PIN = 7;

int16_t CH1WaveBuff, CH1Filterout;
int16_t CH2WaveBuff, CH2Filterout;
int16_t CH3WaveBuff, CH3Filterout;
int16_t CH4WaveBuff, CH4Filterout;

ads1194 ADS1194;
ecg_algorithm ECG_ALGORITHM;

void setup()
{
  delay(2000);

  SPI.begin();
  SPI.setBitOrder(MSBFIRST);
  SPI.setDataMode(SPI_MODE1);
  SPI.setClockDivider(SPI_CLOCK_DIV16);

  pinMode(ADS1194_DRDY_PIN, INPUT);
  pinMode(ADS1194_CS_PIN, OUTPUT);
  pinMode(ADS1194_START_PIN, OUTPUT);
  pinMode(ADS1194_PWDN_PIN, OUTPUT);
  Serial.begin(57600);

  ADS1194.ads1194Init(ADS1194_CS_PIN,ADS1194_PWDN_PIN,ADS1194_START_PIN);
  Serial.println("Initiliziation is done");
 
}

void loop()
{
  ads1194OutputValues ecgvalues;

  boolean ret = ADS1194.getAds1194EcgSamples(ADS1194_DRDY_PIN,ADS1194_CS_PIN,&ecgvalues);
  if (ret == true)
  {
    CH1WaveBuff = (int16_t)(ecgvalues.sDaqVals[0]) ;  
    CH2WaveBuff = (int16_t)(ecgvalues.sDaqVals[1]) ;  
    CH3WaveBuff = (int16_t)(ecgvalues.sDaqVals[2]) ;  
    CH4WaveBuff = (int16_t)(ecgvalues.sDaqVals[3]) ;  
    if(ecgvalues.leadoffDetected == false)
    {
      // Uncomment for CH1
      //ECG_ALGORITHM.ECG_ProcessCurrSample(&CH1WaveBuff, &CH1Filterout);   // filter out the line noise @40Hz cutoff 161 order
      //ECG_ALGORITHM.QRS_Algorithm_Interface(CH1Filterout,&globalHeartRate); // calculate
      // Uncomment for CH2
      ECG_ALGORITHM.ECG_ProcessCurrSample(&CH2WaveBuff, &CH2Filterout);   // filter out the line noise @40Hz cutoff 161 order
      ECG_ALGORITHM.QRS_Algorithm_Interface(CH2Filterout,&globalHeartRate); // calculate
      // Uncomment for CH3
      //ECG_ALGORITHM.ECG_ProcessCurrSample(&CH3WaveBuff, &CH3Filterout);   // filter out the line noise @40Hz cutoff 161 order
      //ECG_ALGORITHM.QRS_Algorithm_Interface(CH3Filterout,&globalHeartRate); // calculate
      // Uncomment for CH4
      //ECG_ALGORITHM.ECG_ProcessCurrSample(&CH4WaveBuff, &CH4Filterout);   // filter out the line noise @40Hz cutoff 161 order
      //ECG_ALGORITHM.QRS_Algorithm_Interface(CH4Filterout,&globalHeartRate); // calculate
    }
    else{
      // Uncomment for CH1
      //CH1Filterout = 0;
      // Uncomment for CH2
      CH2Filterout = 0;
      // Uncomment for CH3
      //CH3Filterout = 0;
      // Uncomment for CH4
      //CH4Filterout = 0;
    }
    //To see the output of another channel on the plotter, change CH1Filterout to CHXFilterout, uncomment the specific channel number lines, and comment out the CH2 lines.
    Serial.println(CH2Filterout);
  }
}
